/*
 *
 * Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Zhenyu Wang <wangzhenyu@kylinos.cn>
 *
 */

#include "../../kysdk-widgetutils/src/kwidgetutils.h"
#include <QtTest>
using namespace kdk;

class WidgetUitls : public QObject
{
    Q_OBJECT

private slots:
    void widgetutilsTest();
};

void WidgetUitls::widgetutilsTest()
{
    KWidgetUtils widget;
    widget.highDpiScaling();
    widget.checkCompositorRunning();
}

QTEST_APPLESS_MAIN(WidgetUitls)
#include "widgetutilstest.moc"
