/*
 * libkysdk-waylandhelper's Library
 *
 * Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Zhen Sun <sunzhen1@kylinos.cn>
 *
 */

#include "blurmanager.h"
#include "ukui-blur-manager-client-protocol.h"

#include <QGuiApplication>
#include <QRegion>
#include <QWindow>

#include "blurwaylandinterface.h"
#include "blurxcbinterface.h"

namespace kdk
{

static BlurManager *g_blurmanager = nullptr;

BlurManager::BlurManager(QObject *parent)
    : QObject(parent)
{
    QString platform = QGuiApplication::platformName();
    if (platform.startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
        m_blurInterface = new BlurWaylandInterface(this);
    else
        m_blurInterface = new BlurXcbInterface(this);
}

BlurAbstractInterface *BlurManager::interface()
{
    return m_blurInterface;
}

BlurManager *BlurManager::self()
{
    if (g_blurmanager)
        return g_blurmanager;
    g_blurmanager = new BlurManager();
    return g_blurmanager;
}

bool BlurManager::setBlurBehindWithStrength(QWindow *window, bool enable, const QRegion &region, uint32_t strength)
{
    return self()->interface()->setBlurBehindWithStrength(window, enable, region, strength);
}

}
