/*
 * libkysdk-waylandhelper's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Zhen Sun <sunzhen1@kylinos.cn>
 *
 */

/* Generated by wayland-scanner 1.18.0 */

#include "wayland-util.h"
#include <stdint.h>
#include <stdlib.h>

#ifndef __has_attribute
#define __has_attribute(x) 0 /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *custom_types[] = {
    &wl_surface_interface,
    &wl_surface_interface,
    NULL,
    &wl_surface_interface,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct wl_message ukui_decoration_requests[] = {
    {"move_surface", "o", custom_types + 0},
    {"set_ukui_decoration_mode", "ou", custom_types + 1},
    {"set_unity_border_radius", "ouuuu", custom_types + 3},
};

WL_PRIVATE const struct wl_interface ukui_decoration_interface = {
    "ukui_decoration",
    1,
    3,
    ukui_decoration_requests,
    0,
    NULL,
};
